@@ -1,3 +1,7 @@
+
+#'
+#' @importFrom stats glm as.formula
+#'
 get.MLR <- function(alpha.P, Y.i, q.vector) {
 
   att <- which(q.vector == 1)
@@ -6,7 +10,7 @@ get.MLR <- function(alpha.P, Y.i, q.vector) {
 
   formula <- paste("Y ~", paste(names(data)[-length(names(data))], collapse = " + "))
 
-  MLR <- stats::glm(stats::as.formula(formula), data = data, family = "binomial")
+  MLR <- glm(as.formula(formula), data = data, family = "binomial")
   coeffs <- summary(MLR)$coefficients
   r <- coeffs[-1, 1]
   p <- coeffs[-1, 4]
